#---------------------------------------------------------------------------------------------------------
#. Universidade Estadual de Santa Cruz  - UESC
#. Tema: Variáveis Aleatórias Discretas - VAD
# Alunos: Brendon Wilyen de Queiroz Souza,
#         Erika Rafaesky Santos,
#         Evelyn dos Santos Silva
# Revisão: 2024-06-29 21:59:11 - J.C.Faria
#---------------------------------------------------------------------------------------------------------

#. Criação de funções customizadas para demonstração
#.. Bernoulli
# x: número de sucessos
# p: probabilidade do sucesso

dbern_custom <- function(x, p)
  p^x * (1-p)^(1-x)

#.. Binomial
# x: número de sucessos
# n: número de ensaios
# p: probabilidade do sucesso

dbinom_custom <- function(x, n, p)
   (factorial(n)/(factorial(x) * factorial(n-x))) *
   p^x * (1-p)^(n-x)
 
#.. Poisson
# Função de probabilidade implementada
dpois_custom <- function(x,
                         lambda)
  (exp(1)^-lambda)*(lambda^x) / factorial(x)

#... Observar: Binomial generaliza a de Bernoulli
# Em que situação?
# Quando (factorial(n)/(factorial(x)* factorial(n - x))) é igual a 1!
# Ou seja, em apenas uma repetição do experimento!

# Verificação:
x <- 1  # número de sucessos (não faz diferença se é 0 ou 1)
n <- 1  # número de tentativas

(factorial(n)/(factorial(x) * factorial(n-x)))

#... Exemplo 1 - Distribuição de Bernoulli e Binomial
# Em uma questão com 5 alternativas, qual a probabilidade de acertar essa 
# questão no chute?
x <- 1
p <- 1/5

# Bernoulli
(res_bern <- dbern_custom(x, p))

# Binomial
(res_dbin <- dbinom_custom(x=x,
                           n=1,
                           p=p))

# A distribuição de Bernoulli é um caso especial da distribuição Binomial, com n = 1.
res_bern == res_dbin

# Verificação
# Sabemos que chutando uma alternativa entre 5 possíveis, a probabilidade de 
# acertar em uma única tentativa é: 1/5 = 0.2
res_bern == 1/5

#... dbinom: nativa do R
str(dbinom)

dbinom(x=x,
       size=1,
       prob=p)

# verificação
dbinom(x=x,
       size=1,
       prob=p) == res_dbin

#... Exemplo 2 - Distribuição Binomial
# Uma moeda é lançada 10 vezes. Qual a probabilidade de ocorrer pelo menos
# 1 coroa?
# Como só não pode ocorrer 0 coroas, temos:
x <- 0    # número de sucessos
n <- 10   # número de tentativas
p <- 1/2  # probabilidade de sucesso

# Binomial customizada
(res_dbin <- dbinom_custom(x,
                           n,
                           p))

# A probabilidade de ocorrer pelo menos uma coroa, são todas as outras probabilidades, 
# menos a de 0 coroas, logo:
# P(S) - P(x=0)
1 - res_dbin

# Usando função nativa do R
# Utilizando ainda o exemplo anterior, podemos calcular agora a probalidade de 
# ocorrer 6 caras em 10 lançamentos de uma moeda:
x <- 6       # número de sucessos
size <- 10   # número de tentativas
prob <- 1/2  # probabilidade de sucesso

# Utilizando a binomial já implementada no R
(verify <- dbinom(x,
                  size,
                  prob))

# Verificação
# A probabilidade de ocorrer 6 caras em 10 lançamentos é de 210/1024 = 0.2050781, 
# ou seja, aproximadamente 21%.
round(verify, 7) == round(210/1024, 7)

# (210/1024)?
ncol(combn(10, 6))
2^10

# Esperança Matemática: E(X)
(E <- p)

# Variância: V(X)
(V <- p*(1-p))

#... Exemplo 3 - Distribuição Binomial
x <- 2    # número de moléculas raras que quero achar (número de sucessos)
n <- 18   # número de tentativas ou experimentos
p <- 0.1  # probabilidade de se encontrar uma molécula rara em uma tentativa

# Probabilidade de se encontrar 2 moléculas em 18 tentativas
dbinom_custom(x,
              n,
              p)

dbinom(x,
       n,
       p)

# Vamos alterar o exemplo
x <- 5    # desejamos encontrar 5 moléculas raras
n <- 10   # alterar o número de tentativas para 10
p <- 0.5  # alterar a probabilidade para 50%

dbinom_custom(x,
              n,
              p) # Probabiliade de se encontrar 5 moléculas em 10 tentativas

dbinom(x,
       n,
       p)

# Vamos agora diminuir o número de sucessos que desejamos encontrar
x <- 2

# A probabilidade vai aumentar ou diminuir?
dbinom_custom(x,
              n,
              p)

dbinom(x,
       n,
       p)

# A probabilidade diminui!

# Esperança Matemática
(E <- n*p)

# Variância
(V <- n*p*(1-p))

#... Exemplo 4 - Distribuição Geométrica
# Uma linha de produção está sendo analisada para efeito de controle de qualidade 
# das peças produzidas. Tendo em vista o alto padrão requerido, a produção é interrompida 
# para regulagem toda vez que uma peça defeituosa é observada, onde a probabilidade da 
# peça ser defeituosa é de 0,01.

# Calcule a probabilidade da 1ª peça ser defeituosa.
x <- 1-1   # peças defeituosas Tentativa k - 1.
p <- 0.01  # probabilidade de ocorrer sucesso na k-ésima tentativa

#.. Geométrica (nativa do R)
(result <- dgeom(x,
                 p))

# Verificação
# Para ocorrer uma peça defeituosa logo na primeira peça fabricada, a possibilidade 
# de fracasso é 0. (Tentativas - 1). Portanto, há somente a possibilidade de sucesso, 
# que é 0.01, dada no exemplo.
result == 0.01

# Esperança Matemática
(E <- 1/p)

# Variância
(V <- (1-p)/p*p) 

#... Exemplo 5 - Distribuição Binomial Negativa
# Calcular o número de tentativas até que ocorram r sucessos. 
# Vamos lançar uma moeda até que saiam 3 caras.
x <- 3-3  # número de fracassos dado pelo número de tentativas - número de sucessos.
r <- 3    # número de sucessos
p <- 1/2  # probabilidade de sucesso

#.. Binomial negativa (nativa do R)
(result<- dnbinom(x,
                  r,
                  p))

# Verificação
# Sabemos que para que saiam 3 caras em 3 tentativas, temos que obter três sucessos 
# consecutivos, portanto a probabilidade é p³, sendo 1/2 a probabilidade de cada 
# tentativa, temos: (1/2)³ = 0.125.
round(result, 3) == round(0.125, 3) # Aproximação utilizando 3 casas decimais.

# Esperança Matemática
(E <- r/p)

# Variância
(V <-  r*(1-p)/p*p)

#... Exemplo 6 - Distribuição Hipergeométrica
# Em uma caixa temos 10 bolas, sendo 4 brancas e 6 vermelhas. Se três são retiradas 
# sem reposição, qual a probabilidade de sair duas bolas vermelhas?
k <- 6       # número de elementos do tipo desejado
N <- 10 - 6  # totalidade de elementos - número de elementos do tipo desejado.
n <- 3       # número de elementos selecionados
x <- 2       # número de sucessos desejados

#.. Hipergeométrica (nativa do R)
(result <- dhyper(x,
                  k,
                  N,
                  n))

# Verificação 
# A partir de contas feitas utilizando a função de probabilidade estudada anteriomente, 
# a probabilidade é igual a 1/2.
round(result, 2) == round(1/2, 2) # Aproximação utilizando 2 casas decimais.

(p <- k/N)

# Esperança Matemática
(E <- n*p)

# Variância
(V <- n*p*(1-p)*((N-n)/(N-1)))

#... Exemplo 7 - Poisson
# Em um fio delgado de cobre, o número de falhas no fio segue a
# distribuição de Poisson, com uma média de 2,3 falhas por
# milímetro. Determine a probabilidade de existir exatamente 2
# falhas em um milímetro de fio.

x <- 2
lambda <- 2.3

dpois_custom(x,
             lambda)

# Função nativa do R
dpois(x,
      lambda)

# Esperança Matemática
(E <- lambda)

# Variância
(V <- lambda)

#... Exemplo 8 - Distribuição de Poisson
# Em média há 2 chamadas por hora em um certo telefone.
# Calcule a probabilidade de receber no máximo 3 chamadas em 2 horas.

# Dados
x <- 0:3
lambda <- 4  # o tempo são 2 horas portanto, lambda = 4

# Poisson custom
(res_c <- dpois_custom(x,
                       lambda))

sum(res_c)   # 0.43

# dpois: Poisson nativa do R
(res_n <-  dpois(x,
                 lambda))

# A probabilidade de se receber no máximo 3 chamadas em 2 horas, é uma soma das
# probabilidades de receber: 0, 1, 2 e 3 chamadas:
sum(res_n)   # 0.43


#. Gráficos das principais VADs
#.. Bernoulli
# Função de probabilidades customizada
dbern_custom <- function(x,
                         p)
  p^x * (1-p)^(1-x)

#... Lançamento de uma moeda
x <- 1    # sucesso: ocorrer cara
p <- 1/2  # probabilidade

# Cálculo da probabilidade
(p_bern <- dbern_custom(x,
                        p))

# Plot
plot(p_bern ~ x,
     pch='*',
     cex=2,
     lwd=2,
     ylim=c(0, 1),
     xaxt='n',
     yaxt='n',
     col='darkred',
     xlab='Lançamento',
     ylab='Probabilidade de Bernoulli',
     main='Distribuição de Bernoulli (moedas)')

# Eixos
axis(1,
     at=1)

axis(2,
     at=c(0, 1/2, 1))

# Esperança
abline(h=p_bern,
       col='darkred',
       lwd=2)

#... Lançamento de um dado
x <- 1    # sucesso: face 3
p <- 1/6  # probabilidade

# Cálculo da probabilidade
(p_bern <- dbern_custom(x,
                        p))

# Plot
plot(p_bern ~ x,
     pch='*',
     cex=2,
     lwd=2,
     ylim=c(0, 1),
     xaxt='n',
     yaxt='n',
     col='darkred',
     xlab='Lançamento',
     ylab='Probabilidade de Bernoulli',
     main='Distribuição de Bernoulli (dados)')

# Eixos
axis(1,
     at=1)

axis(2,
     at=round(c(0, 1/6, 1),
              3))

# Esperança
abline(h=p_bern,
       col='darkred',
       lwd=2)


#.. Binomial
#... Lançamento de uma moeda 10 vezes
x <- 0:10  # vetor do número de sucessos esperados
n <- 10    # número de lançamentos
p <- 1/2   # probailidade de sucesso de uma determinada face

(p_binom <- dbinom(x,
                   n,
                   p))

plot(p_binom ~ x,
     type='b',
     pch='*',
     lwd=2,
     xaxt='n',
     yaxt='n',
     col='darkred',
     xlab='Nº de lançamentos moeda',
     ylab='Probabilidade da Binomial',
     main='Distribuição Binomial')

# Eixos
axis(1,
     at=0:10)

axis(2,
     at=round(p_binom,
              3))
# Esperança
abline(v=mean(n*p),      # E(x) = n*p
       col='darkgreen',
       lwd=2)

#... Lançamento de um dado 10 vezes
x <- 0:10  # vetor do número de sucessos esperados
n <- 10    # número de lançamentos
p <- 1/6   # probailidade de sucesso de uma determinada face

(p_binom <- dbinom(x,
                   n,
                   p))

plot(p_binom ~ x,
     type='b',
     pch='*',
     lwd=2,
     xaxt='n',
     yaxt='n',
     col='darkred',
     xlab='Nº de lançamentos dados',
     ylab='Probabilidade da Binomial',
     main='Distribuição Binomial')

# Eixos
axis(1,
     at=0:10)

axis(2,
     at=round(p_binom,
              3))
# Esperança
abline(v=mean(n*p),      # E(x) = n*p
       col='darkgreen',
       lwd=2)

#... Aproximação da normal pela binomial: Lançamento de um 300 dados
# (ou 300 lançamentos de um dado)
x <- 0:100  # vetor do número de sucessos esperados
n <- 3e2    # número de lançamentos
p <- 1/6    # probailidade de sucesso de uma determinada face

(p_binom <- dbinom(x,
                   n,
                   p))

plot(p_binom ~ x,
     type='b',
     pch='*',
     lwd=2,
     xaxt='n',
     yaxt='n',
     col='darkred',
     xlab='Nº de lançamentos dados',
     ylab='Probabilidade da Binomial',
     main='Distribuição Binomial')

# Eixos
axis(1,
     at=x)

axis(2,
     at=round(p_binom,
              3))
# Esperança
abline(v=mean(n*p),      # E(x) = n*p
       col='darkgreen',
       lwd=2)

text(x=n*p,
     y=.03,
     labels=expression(E(x)))


#.. Poisson
# Probabilidades de erros no intervalo de 0 a 10
x <- 0:10
lambda <- 2.3

(p_pois <- dpois(x,
                 lambda))

plot(pois ~ x,
     type='b',
     pch='*',
     lwd=2,
     xaxt='n',
     yaxt='n',
     col='darkred',
     xlab='Nº de erros por milímetro',
     ylab='Probabilidade de Poisson',
     main='Distribuição de Poisson')

# Eixos
axis(1,
     at=0:10)

axis(2,
     at=round(p_pois,
              3))

# Esperança
abline(v=lambda,      # E(x) = lambda
       col='darkgreen',
       lwd=2)

text(x=lambda,
     y=.1,
     labels=expression(E(x)))
